/*
 * rijndael-icm.h
 *
 * Header for RIJNDAEL Integer Counter Mode.
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 *
 */

#ifndef RIJNDAEL_ICM_H
#define RIJNDAEL_ICM_H

#include "rijndael.h"
#include "cipher.h"

typedef struct {
  v128_t counter;                     /* holds the counter value          */
  v128_t offset;                      /* initial offset value             */
  v128_t keystream_buffer;            /* buffers bytes of keystream       */
  int    bytes_in_buffer;             /* number of unused bytes in buffer */
  rijndael_expanded_key expanded_key; /* the cipher key                   */
} rijndael_icm_context;


err_status_t
rijndael_icm_context_init(rijndael_icm_context *c,
			  const unsigned char key[16],
			  const unsigned char salt[16]); 

err_status_t
rijndael_icm_set_segment(rijndael_icm_context *c,
			 xtd_seq_num_t index);

err_status_t
rijndael_icm_encrypt(rijndael_icm_context *c,
		     unsigned char *buf, int bytes_to_encr);

err_status_t
cipher_set_to_rijndael_icm(cipher_type_t *c, rijndael_icm_context *ctx);


#endif /* RIJNDAEL_ICM_H */

