/*
 * seal.h
 *
 * SEAL (Software-optimized Encryption Algorithm) implementation
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 *
 * Note that SEAL is covered by U.S. Patents.  YOU ARE SOLELY
 * RESPONSIBLE FOR USING THIS CODE IN A MANNER CONSISTENT WITH THE
 * LICENSING TERMS AND CONDITIONS.
 *
 */

/*
 *	
 * Copyright  (c) 2001, 2002, Cisco Systems, Inc.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials provided
 *   with the distribution.
 * 
 *   Neither the name of the Cisco Systems, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef SEAL_H
#define SEAL_H

#include "cipher.h"
#include "datatypes.h"
#include "sha1.h"        /* SEAL uses the SHA-1 compression function */

typedef struct seal_ctx_t {
                                /* key-dependant state           */
  uint32_t R[16];               /* initialization table          */
  uint32_t S[256];              /* output mask                   */
  uint32_t T[512];              /* substitution table            */
                                
                                /* index-dependant state         */
  uint32_t index;               /* 32-bit segment index          */

                                /* stream state                  */
  int l;                        /* round counter                 */
  uint32_t state[4];            /* hold the current cipher state */
  uint32_t n1, n2, n3, n4;      /* used in round function        */
  int bytes_in_buffer;          /* number unused bytes in buffer */
  v128_t keystream;             /* used in buffering output      */
} seal_ctx_t;

/*
 * The state[] array has the following equivalence to the variables
 * defined in the SEAL3.0 specification:
 * 
 * state[0] = B
 * state[1] = C
 * state[2] = D
 * state[3] = A
 */

#endif /* SEAL_H */
