/*
 * tmmh.h
 *
 * the truncated multi-modular (keyed) hash

 * This file is for VERSION ONE of TMMH - you probably want version
 * two, which is in the file tmmhv2.c.
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 */

/*
 *	
 * Copyright  (c) 2001, 2002, Cisco Systems, Inc.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials provided
 *   with the distribution.
 * 
 *   Neither the name of the Cisco Systems, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */



#ifndef TMMH_H
#define TMMH_H

#include "hash.h" 
#include "integers.h"

#define tmmh_16_prime 65537

typedef struct {
  int       key_length;     /* length of hash key in octets      */
  int       tag_length;     /* length of the output in octets    */
  uint16_t *value;
} tmmh_16_ctx_t;

hash_status
tmmh_16_init(tmmh_16_ctx_t *key, octet_t *value, int key_len, int tag_len);

hash_status
tmmh_16_compute(tmmh_16_ctx_t *key, octet_t *msg, int len, octet_t *result);

hash_status
tmmh_16_compute_aligned(tmmh_16_ctx_t *key, uint16_t *msg,
                        int len_octets, uint16_t *result);

hash_status
tmmh_16_compute_4_aligned(tmmh_16_ctx_t *key, uint16_t *msg,
                          int len_octets, uint16_t *result);

hash_status
hash_set_to_tmmh_16(hash *h, tmmh_16_ctx_t *ctx, int output_len);

/* tmmh_32 interface is below */

#define tmmh_32_prime 0x10000000FLL

typedef struct {
  uint32_t *value;
  int       key_length;     /* length of hash key in octets      */
  int       tag_length;     /* length of the output in octets    */
  uint64_t *state;          /* state used in init/update/final   */
  uint32_t *current_key;    /* key pointer for init/update/final */
} tmmh_32_ctx_t;


hash_status
tmmh_32_init(tmmh_32_ctx_t *key, octet_t *value, int key_len, int tag_len);

hash_status
tmmh_32_compute(tmmh_32_ctx_t *key, octet_t *msg, int len, octet_t *result);

hash_status
hash_set_to_tmmh_32(hash *h, tmmh_32_ctx_t *ctx, int out_len);

/* the following is for TEST ONLY */

hash_status
tmmh_16_compute_aligned(tmmh_16_ctx_t *key, uint16_t *msg,
			int len_octets, uint16_t *result);

#endif /* TMMH_H */
