/*
 * tmmhv2.h
 *
 * tmmhv2 is the mmh hash function, with authenticated truncation,
 * version two
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 */

/*
 *	
 * Copyright  (c) 2001, 2002, Cisco Systems, Inc.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials provided
 *   with the distribution.
 * 
 *   Neither the name of the Cisco Systems, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */



#ifndef TMMH_H
#define TMMH_H

#include "auth.h" 
#include "integers.h"

#define tmmh_16_prime 65537

typedef struct {
  uint16_t *value;
} tmmhv2_ctx_t;

err_status_t
tmmhv2_alloc(auth_t **a, int key_len, int out_len);

err_status_t
tmmhv2_dealloc(auth_t *a);

err_status_t
tmmhv2_init(tmmhv2_ctx_t *state, const octet_t *key, int key_len);

err_status_t
tmmhv2_compute (tmmhv2_ctx_t *state, octet_t *message,
		int msg_octets, octet_t *result);


/*
 * tmmhv2_add_big_test()
 * 
 * generates the tmmhv2 large-message test case and adds it to the
 * head of the linked list of test cases
 */

err_status_t
tmmhv2_add_big_test();


#endif /* TMMH_H */


