/*
 * sha1-driver.c
 *
 * a test driver for SHA-1
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 */

/*
 *	
 * Copyright  (c) 2001, 2002, Cisco Systems, Inc.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   disclaimer in the documentation and/or other materials provided
 *   with the distribution.
 * 
 *   Neither the name of the Cisco Systems, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include <stdio.h>
#include "sha1.h"



int
hmac_sha1_test();

int
main () {
  uint32_t hash_value[5];
  uint32_t message[16]  = { 
    0x61626380, 0x00000000, 0x00000000, 0x00000000, 
    0x00000000, 0x00000000, 0x00000000, 0x00000000, 
    0x00000000, 0x00000000, 0x00000000, 0x00000000, 
    0x00000000, 0x00000000, 0x00000000, 0x00000018 
  };
  uint32_t msg_ref[1] = { 0x61626300 };
  uint32_t hash_ref[5] = {
    0xa9993e36, 0x4706816a, 0xba3e2571, 0x7850c26c, 0x9cd0d89d 
  };
  uint32_t msg_ref2[14] = {
    0x61626364, 0x62636465, 0x63646566, 0x64656667,
    0x65666768, 0x66676869, 0x6768696a, 0x68696a6b,
    0x696a6b6c, 0x6a6b6c6d, 0x6b6c6d6e, 0x6c6d6e6f,
    0x6d6e6f70, 0x6e6f7071
  };
  uint32_t hash_ref2[5] = {
    0x84983e44, 0x1c3bd26e, 0xbaae4aa1, 0xf95129e5, 0xe54670f1
  };
    
  printf("sha1 test driver\n"
         "David A. McGrew\n"
         "Cisco Systems, Inc.\n");

  sha1(message, 64, hash_value);

  sha1_x(msg_ref, 3, hash_value);
  printf("hash_value: { %x%x%x%x%x } \n", hash_value[0], hash_value[1],
	 hash_value[2], hash_value[3], hash_value[4]);
  printf("hash_ref:   { %x%x%x%x%x } \n", hash_ref[0], hash_ref[1],
	 hash_ref[2], hash_ref[3], hash_ref[4]);
  

  sha1_x(msg_ref2, 56, hash_value);
  printf("hash_value2: { %x%x%x%x%x } \n", hash_value[0], hash_value[1],
	 hash_value[2], hash_value[3], hash_value[4]);
  printf("hash_ref2:   { %x%x%x%x%x } \n", hash_ref2[0], hash_ref2[1],
	 hash_ref2[2], hash_ref2[3], hash_ref2[4]);


#if THE_HMAC_FUNCTION_HAS_BEEN_IMPLEMENTED_YET  
  hmac_sha1_test();
#endif

  return 0;
}


int
hmac_sha1_test() {
  uint32_t key[5] =  { 0x0b0b0b0b, };
  int key_len = 20;
  unsigned char data[20] = "Hi There";
  int data_len = 8;
  uint32_t tag[5];
  uint32_t ref_tag[5] = { 
    0xb6173186, 0x55057264, 0xe28bc0b6, 0xfb378c8e, 0xf146be00
  };

  printf("testing hmac-sha1...\n");

  hmac_sha1((uint32_t *)data, data_len, key, key_len, tag);

  printf("tag:     { %x%x%x%x%x } \n", tag[0], tag[1],
	 tag[2], tag[3], tag[4]);  
  printf("ref_tag: { %x%x%x%x%x } \n", ref_tag[0], ref_tag[1],
	 ref_tag[2], ref_tag[3], ref_tag[4]);  

  return 0;

}

