/*
 * stat-driver.c
 *
 * test driver for the stat_test functions
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 */


#include <stdio.h>         /* for printf() */

#include "err.h"
#include "stat.h"

#include "cipher.h"

typedef struct {
  void *state;
} random_source_t;

err_status_t
random_source_alloc();

int
main (int argc, char *argv[]) {
  octet_t buffer[2500];
  int i, j;
  extern cipher_type_t rijndael_icm;
  cipher_t *c;
  octet_t key[16], salt[14];
  xtd_seq_num_t idx = { 0x01234567, 0x89ab };
  int num_trials = 10000;
  int num_fail;

  printf("statistical tests driver\nDavid A. McGrew\nCisco Systems, Inc.\n");

  for (i=0; i < 2500; i++)
    buffer[i] = 0;

  /* run tests */
  printf("running stat_tests on all-null buffer, expecting failure\n");
  printf("monobit %d\n", stat_test_monobit(buffer));
  printf("poker   %d\n", stat_test_poker(buffer));
  printf("runs    %d\n", stat_test_runs(buffer));

  for (i=0; i < 2500; i++)
    buffer[i] = random();
  printf("running stat_tests on random(), expecting success\n");
  printf("monobit %d\n", stat_test_monobit(buffer));
  printf("poker   %d\n", stat_test_poker(buffer));
  printf("runs    %d\n", stat_test_runs(buffer));

  /* set buffer to cipher output */
  for (i=0; i < 2500; i++)
    buffer[i] = 0;
  cipher_type_alloc(&rijndael_icm, &c, 16);
  cipher_init(c, key, salt);
  cipher_set_segment(c, idx);
  cipher_encrypt(c, buffer, 2500);

  printf("running stat_tests on AES-128-ICM, expecting success\n");
  printf("monobit %d\n", stat_test_monobit(buffer));
  printf("poker   %d\n", stat_test_poker(buffer));
  printf("runs    %d\n", stat_test_runs(buffer));

  num_fail = 0;
  for(j=0; j < num_trials; j++) {
    for (i=0; i < 2500; i++)
      buffer[i] = 0;
    index_advance(&idx, 1);
    cipher_set_segment(c, idx);
    cipher_encrypt(c, buffer, 2500);
    if (stat_test_runs(buffer)) {
      num_fail++;
    }
  }

  printf("runs test: %d failures in %d tests\n", num_fail, num_trials);
  printf("(nota bene: a small fraction of stat_test failures does not \n"
	 "indicate that the random source is invalid)\n");

  return 0;
}
